/* FifoReporter.c
 * 
 * Copyright (C) 2011 Simone Piccardi
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/****************************************************************
 *
 * Program reporterd
 * FIFO, signal, event and alarm reporter
 * an example program for I/O multiplexing with signalfd and Co.
 *
 * Author: Simone Piccardi
 * Jan. 2011
 *
 * Usage: reporterd -h give all info
 *
 ****************************************************************/
/* 
 * Include needed headers
 */
#include <sys/types.h>   /* primitive system data types */
#include <sys/stat.h>    /* file characteristics constants and functions */
#include <unistd.h>      /* unix standard library */
#include <stdio.h>       /* standard I/O library */
#include <stdlib.h>	 /* C standard library */
#include <string.h>	 /* C strings library */
#include <errno.h>	 /* error definitions and routines */
#include <signal.h>	 /* signal constants, types and functions */
#include <fcntl.h>	 /* file control functions */
#include <sys/epoll.h>   /* Linux epoll interface */

#include "macros.h"
#include "Gapil.h"

/* Subroutines declaration */
void usage(void);

/* default name for the input fifo */
char *fifoname = "/tmp/reporter.fifo";

int main(int argc, char *argv[])
{
/* Variables definition */
    int i, t = 1;
    char buffer[4096];
    int fifofd, epfd;
    struct epoll_event epev;
    int nread;
    /*
     * Input section: decode parameters passed in the calling 
     * Use getopt function
     */
    opterr = 0;	                             /* don't want writing to stderr */
    while ( (i = getopt(argc, argv, "ht:f:")) != -1) {
	switch (i) {
	/* 
	 * Handling options 
	 */ 
	case 'h':  
	    printf("Wrong -h option use\n");
	    usage();
	    return(0);
	    break;
	case 'f':
	    fifoname = optarg;
	    break;
	case 't':
	    t = strtol(optarg, NULL, 10);
	    break;
	case '?':                                    /* unrecognized options */
	    printf("Unrecognized options -%c\n",optopt);
	    usage();
	default:                                       /* should not reached */
	    usage();
	}
    }
    /* ***********************************************************
     * 
     *		 Options processing completed
     *
     *		      Main code beginning
     * 
     * ***********************************************************/
    /* 
     * Initial setup 
     */
    if (mkfifo(fifoname, 0622)) {  // create well known fifo if does't exist 
	if (errno!=EEXIST) {
	    perror("Cannot create well known fifo");
	    exit(1);
	}
    }
    if ((fifo = open(fifoname, O_RDONLY)) < 0) {    // open fifo
	perror("Cannot open read only well known fifo");
	exit(1);
    }
    epfd = epoll_create(5);       // initialize epoll
    if (epfd < 0) {
        perror("Failing on epoll_create");
        exit(1);
    }
    /* Main body: loop over requests */
    while (1) {
	
    }
    debug("Exiting for unknown reasons\n");
}
/*
 * routine to print usage info and exit
 */
void usage(void) {
    printf("Elementary fortune server\n");
    printf("Usage:\n");
    printf("  fortuned [-h] [-f] -n XXX \n");
    printf("  -h   print this help\n");
    printf("  -t filename   set fifo file\n");
    printf("  -n XXX        set pool depth\n");
    exit(1);
}
/*
 * Signal Handler to manage termination
 */
void HandSIGTERM(int signo) {
    debug("Terminated by %s\n", strsignal(signo));
    unlink(fifoname);
    exit(0);
}
